package com.ruoyi.app.base;

import com.auth0.jwt.JWT;
import com.auth0.jwt.algorithms.Algorithm;
import com.ruoyi.app.domain.TbAppUser;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.ip.AddressUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import eu.bitwalker.useragentutils.UserAgent;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import io.jsonwebtoken.SignatureAlgorithm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;


/**
 * @author Alex
 */
@Service("AppTokenService")
public class AppTokenService {

    /**令牌自定义标识*/
    @Value("${token.header}")
    private String header;
    /**令牌秘钥*/
    @Value("${token.secret}")
    private String secret;
    /**令牌有效期（默认7天）*/
    @Value("${token.expireTime}")
    private int expireTime;

    protected static final long MILLIS_SECOND = 1000;
    protected static final long MILLIS_MINUTE = 60 * MILLIS_SECOND;
    private static final Long MILLIS_MINUTE_TEN = 20 * 60 * 1000L;

    @Autowired
    private RedisCache redisCache;

    /**
     * 获取用户身份信息
     *
     * @return 用户信息
     */
    public AppLoginUser getLoginUser() {
        HttpServletRequest request = ServletUtils.getRequest();
        // 获取请求携带的令牌
        String token = getToken(request);
        if (StringUtils.isNotEmpty(token)) {
            Claims claims = parseToken(token);
            // 解析对应的用户信息
            String mobile = (String) claims.get(Constants.APP_LOGIN_USER_KEY);
            String salt = (String) claims.get("salt");
            String userKey = getTokenKey(mobile);
            AppLoginUser user = redisCache.getCacheObject(userKey);
            if (!salt.equals(user.getSalt())){
                return null;
            }
            return user;
        }
        return null;
    }
    /**
     * 获取用户身份信息
     *
     * @return 用户信息
     */
    public AppLoginUser getLoginUser(String token) {
        if (StringUtils.isNotEmpty(token)) {
            Claims claims = parseToken(token);
            // 解析对应的用户信息
            String mobile = (String) claims.get(Constants.APP_LOGIN_USER_KEY);
            String salt = (String) claims.get("salt");
            String userKey = getTokenKey(mobile);
            AppLoginUser user = redisCache.getCacheObject(userKey);
            if (user == null) {
                return null;
            }
            if (!salt.equals(user.getSalt())){
                return null;
            }
            return user;
        }
        return null;
    }
    /**
     * 设置用户身份信息
     */
    public void setLoginUser(AppLoginUser loginUser) {
        if (StringUtils.isNotNull(loginUser) && StringUtils.isNotEmpty(loginUser.getUser().getMobile())) {
            refreshToken(loginUser);
        }
    }
    /**
     * 删除用户身份信息
     */
    public void delLoginUser(String mobile) {
        if (StringUtils.isNotEmpty(mobile)) {
            String userKey = getTokenKey(mobile);
            redisCache.deleteObject(userKey);
        }
    }
    /**
     * 创建令牌
     *
     * @param loginUser 用户信息
     * @return 令牌
     */
    public String createToken(AppLoginUser loginUser) {
        String salt = IdUtils.fastUUID();
        loginUser.setSalt(salt);
        setUserAgent(loginUser);
        refreshToken(loginUser);

        Map<String, Object> claims = new HashMap<>(1);
        claims.put(Constants.APP_LOGIN_USER_KEY, loginUser.getUser().getMobile());
        claims.put("salt", salt);
        return createToken(claims);
    }
    /**
     * 验证令牌有效期，相差不足20分钟，自动刷新缓存
     *
     * @param loginUser
     * @return 令牌
     */
    public void verifyToken(AppLoginUser loginUser) {
        long expireTime = loginUser.getExpireTime();
        long currentTime = System.currentTimeMillis();
        if (expireTime - currentTime <= MILLIS_MINUTE_TEN) {
            refreshToken(loginUser);
        }
    }
    /**
     * 刷新令牌有效期
     *
     * @param loginUser 登录信息
     */
    public void refreshToken(AppLoginUser loginUser)
    {
        loginUser.setLoginTime(System.currentTimeMillis());
        loginUser.setExpireTime(loginUser.getLoginTime() + expireTime * MILLIS_MINUTE);
        // 根据mobile将loginUser缓存
        String userKey = getTokenKey(loginUser.getUser().getMobile());
        redisCache.setCacheObject(userKey, loginUser, expireTime, TimeUnit.MINUTES);
    }
    /**
     * 设置用户代理信息
     *
     * @param loginUser 登录信息
     */
    public void setUserAgent(AppLoginUser loginUser) {
        UserAgent userAgent = UserAgent.parseUserAgentString(ServletUtils.getRequest().getHeader("User-Agent"));
        String ip = IpUtils.getIpAddr(ServletUtils.getRequest());
        loginUser.setIpaddr(ip);
        loginUser.setLoginLocation(AddressUtils.getRealAddressByIP(ip));
        loginUser.setBrowser(userAgent.getBrowser().getName());
        loginUser.setOs(userAgent.getOperatingSystem().getName());
    }
    /**
     * 从数据声明生成令牌
     *
     * @param claims 数据声明
     * @return 令牌
     */
    private String createToken(Map<String, Object> claims) {
        String token = Jwts.builder()
                .setClaims(claims)
                .signWith(SignatureAlgorithm.HS512, secret).compact();
        return token;
    }

    /**
     * 从令牌中获取数据声明
     *
     * @param token 令牌
     * @return 数据声明
     */
    private Claims parseToken(String token) {
        return Jwts.parser()
                .setSigningKey(secret)
                .parseClaimsJws(token)
                .getBody();
    }

    /**
     * 从令牌中获取用户名
     *
     * @param token 令牌
     * @return 用户名
     */
    public String getUsernameFromToken(String token) {
        Claims claims = parseToken(token);
        return claims.getSubject();
    }

    /**
     * 获取请求token
     *
     * @param request
     * @return token
     */
    private String getToken(HttpServletRequest request) {
        String token = request.getHeader(header);
        if (StringUtils.isNotEmpty(token) && token.startsWith(Constants.TOKEN_PREFIX))
        {
            token = token.replace(Constants.TOKEN_PREFIX, "");
        }
        return token;
    }

    private String getTokenKey(String mobile) {
        return Constants.APP_LOGIN_TOKEN_KEY + mobile;
    }
}
